import datetime
import json
import logging
import re
import shutil
import time

from chibi.atlas import Chibi_atlas_default
from bs4 import BeautifulSoup
from chibi.file import Chibi_path
from chibi.atlas import Chibi_atlas
from .others import metadata
from .regex import banned_time

from chibi_dl.site.base.site import Site
from chibi_requests import Chibi_url
#from .image import Image
from .regex import re_episode
from chibi_dl.site.base.exceptions import Max_retries_reach


logger = logging.getLogger( "chibi_dl.sites.ehentai.episode" )


class Episode( Site ):

    @property
    def id( self ):
        return self.url.path.split( '/' )[2]

    @property
    def token( self ):
        return self.url.base_name

    @classmethod
    def can_proccess( cls, url ):
        if re_episode.match( str( url ) ):
            return cls( url )

    @property
    def cover( self ):
        return self.info.cover_url

    @property
    def images( self ):
        raise NotImplementedError
        return self.info.images

    @property
    def upload_at( self ):
        return self.info.upload_at

    def parse_info( self ):
        return self.metadata

    def parse_metadata( self ):
        meta = self.get_metadata()
        meta.scan_at = datetime.datetime.utcnow()
        meta.url = self.url
        meta.tags = self._parse_tags( meta )
        return meta

    def _parse_tags( self, metadata ):
        tags = Chibi_atlas_default( default_factory=list )
        for tag in metadata.tags:
            if ':' in tag:
                tag, value = tag.split( ':' )
                if tag == 'parody':
                    tag = 'parodie'
                tags[ f"{tag}s" ].append( value )
            else:
                tags[ 'tags' ].append( tag )
        return tags

    def _parse_cover( self ):
        container = self.soup.find( "div", id="bigcontainer" )
        url = Chibi_url( container.img.attrs[ "data-src" ] )
        # url += self.url.session
        return url

    def _parse_images( self ):
        container = self.soup.find( "div", id="thumbnail-container" )
        return [
            Image( self.url + a.attrs[ 'href' ] )
            for a in container.find_all( "a" ) ]

    def download( self, path ):
        raise NotImplementedError()
        if path.is_a_file:
            raise NotImplementedError(
                "no se a implementado la descarga cuando path es un archivo" )
        logger.info(
            f"inicia descarga de {len(self.images)} de {self.info.title}" )

        for image in self.images:
            image.download( path )
        return path

    def compress( self, path_output, path_input, format="zip" ):
        if path_output.is_a_folder:
            path_output += self.info.title
        logger.info( f"comprimiendo con {format}" )
        logger.info( f'comprimiendo en "{path_output}"' )
        result = Chibi_path( shutil.make_archive(
            path_output, format, path_input ) )
        expected = result.replace_extensions( "cbz" )
        result.move( expected )
        return expected

    @property
    def file_name( self ):
        return "{}.{}".format( self.number, "cbz" )

    def get_metadata( self, retries=0, max_retries=5 ):
        if retries >= max_retries:
            raise Max_retries_reach( self, metadata )
        self.wait( 1.5 )
        try:
            meta = metadata.post( json={
                "method": "gdata",
                "gidlist": [
                    [ self.id, self.token ]
                ],
                "namespace": 1
            } )
            result = meta.native
        except json.decoder.JSONDecodeError:
            match = banned_time.search( meta.native.text )
            if match:
                minutes, seconds = match.groups()
                minutes, seconds = int( minutes ), int( seconds ) + 1
                logger.warning( f"banned en {metadata} por {minutes}:{seconds}" )
                time.sleep( minutes * 60 + seconds )
                return self.get_metadata(
                    retries=retries + 1, max_retries=max_retries )
        return result



import copy
import logging
import time

#import cfscrape
import requests

from .exceptions import Max_retries_reach, Cannot_pass_cloud_flare
from chibi_requests import Chibi_url

from selenium import webdriver
from selenium.webdriver.firefox.options import Options
from urllib.parse import urlparse, urlencode, parse_qs, urlunparse

from selenium.webdriver import DesiredCapabilities
from selenium.webdriver.common.by import By
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import undetected_chromedriver as uc


logger = logging.getLogger( "chibi_dl.sites.base.site" )


class Site:
    url = None
    default_user_agent = (
        'Mozilla/5.0 (Windows NT 6.1; WOW64) '
        'AppleWebKit/537.36 (KHTML, like Gecko) '
        'Chrome/56.0.2924.87 Safari/537.36'
    )

    def __init__( self, url=None, *args, parent=None, **kw ):
        if url is None:
            url = self.url
        if not isinstance( url, Chibi_url ):
            url = Chibi_url( url.strip() )

        self.url = url
        self.enable_full_scan = False

        self.urls = []
        self.processing_order = []
        self.parent = parent

        for k, v in kw.items():
            setattr( self, k, v )

        if self.default_user_agent:
            self.user_agent = self.default_user_agent

        self._cloud_flare_passed = False

    @property
    def domain( self ):
        parts = self.url.parts
        parts[2] = ''
        parts[3] = ''
        parts[4] = ''
        url = Chibi_url( urlunparse( parts ) )
        url.session = self.session
        return url

    def build_url( self, url=None, params=None, headers=None, **kw ):
        if url is None:
            url = self.url
        if isinstance( url, str ):
            url = Chibi_url( url )
        url.session = self.session
        if params:
            url = url + params
        if headers:
            headers = dict( **url.headers, **headers )
            url._headers = headers
        return url

    def get(
            self, *args, url=None, delay=10, retries=0, max_retries=5,
            verify=True, timeout=500, **kw ):
        url = self.build_url( url=url, **kw )
        try:
            response = url.get( verify=verify, timeout=timeout )
        except requests.ConnectionError:
            if retries > max_retries:
                logger.exception( "maximo numero de reintentos para {url}" )
                raise
            logger.warning( (
                f"no se pudo connectar con el servicor "
                f"esperando {delay} segundos en {url}" ) )
            time.sleep( delay )
            return self.get(
                *args, url=url, delay=delay, retries=retries, **kw )
        if response.status_code not in self.status_code_ok:
            if retries > max_retries:
                logger.error( "maximo numero de reintentos para {url}" )
                raise Max_retries_reach( self, url )
            if response.status_code == 429:
                logger.warning( (
                    f"status code :{response.status_code} url: {url} "
                    f"se reintentara en 300" ) )
                time.sleep( 300 )
            else:
                logger.warning( (
                    f"status code :{response.status_code} url: {url} "
                    f"se reintentara en {delay}" ) )
                time.sleep( delay )
            return self.get(
                *args, url=url, delay=delay, retries=retries + 1, **kw )
        return response

    @property
    def status_code_ok( self ):
        return [ 200 ]

    @property
    def info( self ):
        try:
            return self._info
        except AttributeError:
            self._info = self.parse_info()
            return self._info

    @property
    def metadata( self ):
        try:
            return self._metadata
        except AttributeError:
            self._metadata = self.parse_metadata()
            return self._metadata

    @property
    def soup( self ):
        try:
            return self._response.native
        except AttributeError:
            self.load()
            return self._response.native

    def append( self, url ):
        url = Chibi_url( url )

        for proccesor in self.processing_order:
            result = proccesor.can_proccess( url )
            if result:
                self.urls.append( result )
                return result

    def __iter__( self ):
        for url in self.urls:
            yield url

    @classmethod
    def from_info( cls, url, data ):
        raise NotImplementedError( "no implementada la funcion from_info" )

    def load( self ):
        response = self.get()
        self._response = response

    def parse_info( self ):
        raise NotImplementedError(
            "no implementada la funcion de parseo de info" )

    def parse_metadata( self ):
        raise NotImplementedError(
            "no implementada la funcion de parseo de metadata" )

    def download( self, path ):
        raise NotImplementedError(
            "no implementada la funcion de download" )

    def compress( self, path_output, path_input, format ):
        raise NotImplementedError(
            "no implementada la funcion de compress" )

    def _pre_to_dict( self ):
        return dict( url=self.url, )

    @classmethod
    def from_site( cls, site, **kw ):
        site_dict = site._pre_to_dict()
        site_dict.update( kw )
        return cls( **site_dict, parent=site )

    def __del__( self ):
        if hasattr( self, '_session' ) and not self.parent:
            logger.info( f"cerrando session de '{self!r}'" )
            self.session.close()
        if hasattr( self, '_browser' ):
            logger.info( "cerrando el navegador" )
            self.browser.quit()

    @property
    def session( self ):
        try:
            return self._session
        except AttributeError:
            if self.parent is None:
                self.build_session()
            else:
                self._session = self.parent.session
            return self._session

    @property
    def user_agent( self ):
        return self.session.headers[ 'User-Agent' ]

    @user_agent.setter
    def user_agent( self, value ):
        self.session.headers[ 'User-Agent' ] = value

    def build_session( self ):
        self._session = requests.session()
        # self._session = cfscrape.create_scraper( self._session )

    def wait( self, seconds=1 ):
        time.sleep( seconds )

    def login( self ):
        pass

    @property
    def firefox( self ):
        return self.browser
        """
        if hasattr( self, 'parent' ) and self.parent:
            return self.parent.firefox
        try:
            return self._firefox
        except AttributeError:
            logger.info( "abriendo firefox" )
            options = self.build_firefox_options()
            self._firefox = webdriver.Firefox( options=options )
            return self._firefox
        """

    @property
    def browser( self ):
        if self.parent:
            return self.parent.browser
        try:
            return self._browser
        except AttributeError:
            options = Options()
            desire_capabilities = DesiredCapabilities.CHROME
            desire_capabilities[ 'pageLoadStrategy' ] = 'eager'
            logger.info( "iniciando chrome" )
            a = uc.Chrome( desired_capabilities=desire_capabilities )
            self._browser = a
            return self._browser

    @property
    def _has_browser_init( self ):
        has_browser = hasattr( self, '_browser' )
        if not has_browser and self.parent:
            return self.parent._has_browser_init
        return has_browser

    def build_firefox_options( self ):
        options = Options()
        options.headless = True
        return options

    @property
    def cookies( self ):
        if self.parent:
            return self.parent.cookies
        try:
            return self._cookies
        except AttributeError:
            return None

    @cookies.setter
    def cookies( self, value ):
        if self.parent:
            self.parent.cookies = value
        else:
            self._cookies = {
                cookie[ 'name' ]: cookie[ 'value' ] for cookie in value }
            self.session.cookies.clear()
            for k, v in self._cookies.items():
                self.session.cookies.set( k, v )

    """
    def get(
            self, *args, delay=16, retries=0, max_retries=5,
            ignore_status_code=None, **kw ):
        try:
            response = self.session.get( *args, **kw )
        except requests.ConnectionError:
            time.sleep( delay )
            logger.warning( (
                "no se pudo connectar con el servicor esperando {} segundos "
                "en {}" ).format( delay, args[0], ) )
            return self.get(
                *args, delay=delay ** 2, retries=retries + 1, **kw )
        if ignore_status_code and response.status_code in ignore_status_code:
            logger.warning( (
                "se recibio {} se ignora" ).format( response.status_code, ) )
            return response
        if response.status_code not in [ 200 ]:
            if retries >= max_retries:
                raise Max_retries_reach
            logger.warning( (
                "se recibio {} ( {} ) esperando {} segundos "
                "en {}" ).format(
                    response.status_code,
                    requests.status_codes._codes[ response.status_code ][0],
                    delay, args[0], ) )
            time.sleep( delay )
            return self.get(
                *args, delay=delay ** 2, retries=retries + 1, **kw )
        return response
    """

    @property
    def cloud_flare_passed( self ):
        return (
            self._cloud_flare_passed
            or ( self.parent and self.parent.cloud_flare_passed ) )

    @cloud_flare_passed.setter
    def cloud_flare_passed( self, value ):
        self._cloud_flare_passed = value
        if self.parent:
            self.parent.cloud_flare_passed = value

    def cross_cloud_flare( self, delay=2 ):
        if not self.cloud_flare_passed:
            logger.info( f"obteniendo {self.url} usando el navegador" )
            self.browser.get( self.url )
            logger.info( f"esperando {delay} segundos a que pase cloud flare" )
            time.sleep( delay )

            self.cookies = self.firefox.get_cookies()
            self.user_agent = self.firefox.execute_script(
                "return navigator.userAgent;" )

            if self.get( url=self.url ).ok:
                logger.info(
                    "se puede usar requests con las cookies "
                    "del navegador para pasar cloudflare" )
                self.cloud_flare_passed = True
            else:
                raise Cannot_pass_cloud_flare

    @property
    def is_browser_open( self ):
        parent = self.parent
        while parent:
            browser = hasattr( parent, '_browser' )
            if browser:
                return True
            parent = parent.parent
        return hasattr( self, '_browser' )

    def build_link( self, href, domain=None ):
        if href.startswith( '/' ):
            if not domain:
                domain = self.domain
            return domain + href
        else:
            return self.build_url( href )

    @property
    def links( self ):
        soup = self.soup
        a = soup.find_all( 'a' )
        for link in a:
            href = link.get( 'href' )
            yield self.build_link( href )



import re


re_show = re.compile( r'https://mangaplus.shueisha.co.jp/titles/\d*' )

re_chapter = re.compile( r'/chapter/([0-9]+)/' )
re_chapter_title = re.compile( r'#[0-9]+"([^*]+)\*' )
re_title = re.compile( r'#MANGA_Plus ([^@]+)@' )



import logging

from .exceptions import Cannot_login
from .regex import re_show, re_video, re_lenguage, re_csrf_token
from chibi_dl.site.base.site import Site as Site_base
from chibi_requests import Chibi_url


logger = logging.getLogger( "chibi_dl.sites.crunchyroll" )


class Site( Site_base ):
    @property
    def token( self ):
        if self.parent:
            return self.parent.token
        return self._token

    @token.setter
    def token( self, value ):
        if self.parent:
            self.parent._token = value
        self._token = value

    def _pre_to_dict( self ):
        return dict(
            url=self.url, user=self.user, password=self.password,
            lenguage=self.lenguage, quality=self.quality )


class Crunchyroll( Site ):
    def __init__( self, *args, **kw ):
        super().__init__( '', *args, **kw )
        from .serie import Serie
        self.processing_order = [ Serie ]

    def append( self, url ):
        from .serie import Serie
        lenguage = re_lenguage.search( url )
        url = Chibi_url( url )
        if not lenguage:
            logger.error(
                "no se encontro el lenguaje en la url {}".format( url ) )
        lenguage = lenguage.group( 1 )
        self.lenguage = lenguage

        if re_show.match( url ):
            self.urls.append( Serie(
                url, user=self.user, password=self.password, parent=self,
                lenguage=lenguage, quality=self.quality ) )
            return True
        elif re_video.match( self.url ):
            raise NotImplementedError
        else:
            logger.error( "no se pudo identificar el tipo de la url" )
        return False

    def build_firefox_options( self ):
        options = super().build_firefox_options()
        options.headless = False
        return options

    def login( self ):
        login_url = Chibi_url( "https://www.crunchyroll.com/{country}/login" )
        headers = {
            'Referer': login_url,
        }
        login_url = login_url.format( country=self.lenguage, headers=headers )
        self.firefox.get( login_url )
        input( "preciona enter cuando termines de logearte" )


        self.cookies = self.firefox.get_cookies()
        self.user_agent = self.firefox.execute_script(
            "return navigator.userAgent;" )

        login_url.session = self.session
        login_response = login_url.get()
        logger.info( "intentanto logearse" )

        if login_response.ok:
            csrf_token =  login_response.native.find(
                'input', id='login_form__token' ).attrs[ 'value' ]
            """
            initial_cookies = self.session.cookies
            csrf_token = re_csrf_token.search(
                initial_page_fetch.text ).group( 1 )
            """

            payload = {
                'login_form[name]': self.user,
                'login_form[password]': self.password,
                'login_form[redirect_url]': '/',
                'login_form[_token]': csrf_token,
            }

            login_post_response = login_url.post( data=payload )
            logger.info( "termino de logerase" )
            """
            response = self.get(
                url=self.url, headers=headers,
                cookies=initial_cookies )
            """
            self.session.headers.update( {
                'Upgrade-Insecure-Requests': '1',
                'Accept-Encoding': 'gzip, deflate'
            } )
        else:
            raise Cannot_login(
                "no se pudo conectar a la pagina de loging",
                status_code=login_response.status_code )

    def i_can_proccess_this( self, url ):
        return re_show.match( url )



import time
import logging

from .regex import re_main, banned_time, banned_time_2
from chibi_dl.site.base.site import Site as Site_base
from chibi_requests import Chibi_url
from chibi.metaphors import Book
from chibi.metaphors.book import End_book
from chibi.atlas import Chibi_atlas


logger = logging.getLogger( "chibi_dl.sites.ehentai" )


class Site( Site_base ):
    def _pre_to_dict( self ):
        return dict(
            url=self.url, user=self.user, password=self.password,
            lenguage=self.lenguage, quality=self.quality )


class Ehentai( Site ):
    def __init__( self, url=None, *args, book=None, **kw ):
        self.urls = []
        if url is None:
            url = Chibi_url( 'https://e-hentai.org/' )
        self.url = url
        self.enable_full_scan = False
        self.processing_order = [ self.episode_class ]

    def append( self, url ):
        result = super().append( url )
        if result:
            return result
        url = Chibi_url( url )

        if re_main.match( str( url.url ) ):
            self.enable_full_scan = True
        return result

    def __iter__( self ):
        if self.urls:
            for url in self.urls:
                time.sleep( 1 )
                yield url
        if self.enable_full_scan:
            for galerie in self.info.galeries:
                time.sleep( 1 )
                yield galerie
            for page in self.pages:
                for galerie in page.info.galeries:
                    time.sleep( 1 )
                    yield galerie

    def parse_info( self ):
        table = self.soup.find(
            'table', { 'class': 'itg gltc' } )
        rows = table.find_all( 'td', { "class": "gl3c glname" } )
        links = [
            self.episode_class( r.a.attrs[ 'href' ], parent=self )
            for r in rows ]
        return Chibi_atlas( galeries=links )

    @property
    def pages( self ):
        book = Book(
            total_elements=self.last_page, page_size=1,
            page=self.current_page,
            offset_dict={ 'page': 'page', 'page_size': 'page_size' } )
        while True:
            try:
                book.next()
                page = type( self )( self.url + book )
                yield page
            except End_book:
                break
    @property
    def current_page( self ):
        return int( self.url.params.get( "page", 1 ) )

    @property
    def last_page( self ):
        page = self.soup.find( 'table', { 'class': 'ptt' } )
        links = page.find_all( 'a' )
        max_page = 0
        for a in links:
            url = Chibi_url( a.attrs[ 'href' ] )
            number = int( url.params.get( 'page', 0 ) )
            if number > max_page:
                max_page = number
        return max_page

    @property
    def episode_class( self ):
        from .episode import Episode
        return Episode

    @classmethod
    def can_proccess( cls, url ):
        if re_main.match( str( url ) ):
            return cls( url )

    def get( self, *args, **kw ):
        response = super().get( *args, **kw )
        match = banned_time.search( response.native.text )
        if match:
            minutes, seconds = match.groups()
            minutes, seconds = int( minutes ), int( seconds ) + 1
            logger.warning( f"banned en {self.url} por 00:{minutes}:{seconds}" )
            time.sleep( minutes * 60 + seconds )
        match = banned_time_2.search( response.native.text )
        if match:
            hours, minutes = match.groups()
            hours, minutes = int( hours ), int( minutes ) + 1
            logger.warning( f"banned en {self.url} por {hours}:{minutes}:00" )
            time.sleep( hours * 60 * 60 + minutes * 60 )
        return response

    def __bool__( self ):
        return bool( self.url )



import re


re_lenguage = re.compile( r'.+/([a-z]{2})/.+' )

re_show = re.compile(
    r'https?://(?:(?P<prefix>www|m)\.)?(?P<url>crunchyroll\.com/[a-z]{2}/'
    '(?!(?:news|anime-news|library|forum|launchcalendar|lineup|store|comics'
    '|freetrial|login))(?P<id>[\w\-]+))/?(?:\?|$)' )
re_video = re.compile(
    r'https?:\/\/(?:(?P<prefix>www|m)\.)?(?P<url>crunchyroll\.'
    '(?:com|fr)/[a-z]{2}/(?:media(?:-|/\?id=)|[^/]*/[^/?&]*?)'
    '(?P<video_id>[0-9]+))(?:[/?&]|$)' )

re_csrf_token = re.compile( r'login_form\[\_token\]\"\ value\=\"(.*?)\"' )


re_episodes = re.compile( r'\<a href\=\"\/(.*?)\"\ title\=\"(.*?)' )



from chibi.config import configuration, Configuration


class Crunchyroll( Configuration ):
    @property
    def video_quality( self ):
        return self[ 'video_quality' ]

    @video_quality.setter
    def video_quality( self, value ):
        if value == 1080:
            value = 80
        elif value == 720:
            value = 62
        elif value == 480:
            value = 61
        elif value == 360:
            value = 60
        elif value == 240:
            value = 60
        else:
            raise NotImplementedError
        self[ 'video_quality' ] = value

    @property
    def resolution( self ):
        return self[ 'resolution' ]

    @resolution.setter
    def resolution( self, value ):
        if value == 1080:
            value = ( 1920, 1080 )
        elif value == 720:
            value = ( 1280, 720 )
        elif value == 480:
            value = ( 848, 480 )
        elif value == 360:
            value = ( 640, 360 )
        elif value == 240:
            value = ( 428, 240 )
        else:
            raise NotImplementedError
        self[ 'resolution' ] = value


configuration.chibi_dl.crunchyroll = Crunchyroll()
configuration.chibi_dl.crunchyroll.resolution = 240
configuration.chibi_dl.crunchyroll.video_format = 108



import re


#https://e-hentai.org/g/618395/0439fa3666/
re_episode = re.compile( r'https://e-hentai.org/g/\d+/\w+/?' )
#re_image = re.compile( r'https://nhentai.net/g/\d+/\d+/?' )
re_main = re.compile( r'https://e-hentai.org/?' )

banned_time = re.compile(
    r'Your IP address has been temporarily banned for excessive '
    r'pageloads which indicates that you are using automated '
    r'mirroring/harvesting software. '
    r'The ban expires in (\d+) minutes and (\d+) seconds' )


banned_time_2 = re.compile(
    r'Your IP address has been temporarily banned for excessive '
    r'pageloads which indicates that you are using automated '
    r'mirroring/harvesting software. '
    r'The ban expires in (\d+) hours and (\d+) minutes' )



import datetime
import logging
import re
import shutil

from bs4 import BeautifulSoup
from chibi.file import Chibi_path
from chibi.atlas import Chibi_atlas

from chibi_dl.site.base.site import Site
from chibi_requests import Chibi_url
from .image import Image
from .regex import re_episode


logger = logging.getLogger( "chibi_dl.sites.ehentai.episode" )


class Episode( Site ):
    @classmethod
    def can_proccess( cls, url ):
        if re_episode.match( str( url ) ):
            return cls( url )

    @property
    def cover( self ):
        return self.info.cover

    @property
    def images( self ):
        return self.info.images

    @property
    def upload_at( self ):
        return self.info.upload_at

    def parse_info( self ):
        info = self.soup.find( "div", id="info" )
        tags = self._parse_tags( info )
        cover = self._parse_cover()
        images = self._parse_images()
        upload_at = datetime.datetime.fromisoformat(
            self.soup.time.attrs[ "datetime" ] )

        return Chibi_atlas(
            title=info.h1.text, tags=tags, cover=cover, images=images,
            upload_at=upload_at )

    def parse_metadata( self ):
        return Chibi_atlas(
            cover_url=self.cover,
            #images_urls=[ image.image for image in self.images ],
            images_len = len( self.images ),
            scan_at=datetime.datetime.utcnow(),
            tags=self.info.tags,
            title=self.info.title,
            upload_at=self.upload_at,
            url=self.url,
            id=self.url.base_name,
        )

    def _parse_tags( self, info ):
        tags = {}
        for section in info.find_all( "div", { "class": "tag-container" } ):
            name = section.find( text=True, recursive=False )
            name = name.strip().lower().replace( ':', '' )
            tags[ name ] = []
            for a in section.find_all( 'a' ):
                text = a.find( text=True )
                text = text.strip()
                tags[ name ].append( text )
        return tags

    def _parse_cover( self ):
        container = self.soup.find( "div", id="bigcontainer" )
        url = Chibi_url( container.img.attrs[ "data-src" ] )
        # url += self.url.session
        return url

    def _parse_images( self ):
        container = self.soup.find( "div", id="thumbnail-container" )
        return [
            Image( self.url + a.attrs[ 'href' ] )
            for a in container.find_all( "a" ) ]

    def download( self, path ):
        if path.is_a_file:
            raise NotImplementedError(
                "no se a implementado la descarga cuando path es un archivo" )
        logger.info(
            f"inicia descarga de {len(self.images)} de {self.info.title}" )

        for image in self.images:
            image.download( path )
        return path

    def compress( self, path_output, path_input, format="zip" ):
        if path_output.is_a_folder:
            path_output += self.info.title
        logger.info( f"comprimiendo con {format}" )
        logger.info( f'comprimiendo en "{path_output}"' )
        result = Chibi_path( shutil.make_archive(
            path_output, format, path_input ) )
        expected = result.replace_extensions( "cbz" )
        result.move( expected )
        return expected

    @property
    def file_name( self ):
        return "{}.{}".format( self.number, "cbz" )



from .base.site import Site


__all__ = [ 'Base' ]


"""
class Site:
    def __init__( self, user, password, **kw ):
        #self.crunchytoll = Crunchyroll( user=user, password=password, **kw )
        #self.tmo_fans = TMO_fans( user=user, password=password, **kw )
        self._sites = [ self.crunchytoll, self.tmo_fans ]

    def append( self, *urls ):
        for url in filter( lambda x: x, urls ):
            site = self.identify_site( url )
            site.append( url )

    def identify_site( self, url ):
        for site in self._sites:
            if site.i_can_proccess_this( url ):
                return site

    def download( self, path ):
        if self.crunchytoll.series:
            self.crunchytoll.login()

        for serie in self.crunchytoll.series:
            serie.download( path )

        for serie in self.tmo_fans.series:
            serie.download( path )

    def print( self, only_links ):
        if self.crunchytoll.series:
            self.crunchytoll.login()

        if only_links:
            self.print_links()
            return

        for serie in self.crunchytoll.series:
            print( serie.name, len( serie.episodes ), serie.url )
            for episode in serie.episodes:
                print( "\t", episode.name, episode.title )

        for serie in self.tmo_fans.series:
            print( serie.name, len( serie.episodes ), serie.url )

    def print_links( self ):
        for serie in self.crunchytoll.series:
            print( serie.url )

        for serie in self.tmo_fans.series:
            print( serie.url )
"""



import logging

from chibi.request import Chibi_url

from .episode import Episode
from .regex import re_chapter, re_chapter_title, re_title
from chibi_dl.site.base.site import Site


logger = logging.getLogger( "chibi_dl.sites.manga_plus.serie" )


class Serie( Site ):
    def __init__( self, url, *args, **kw ):
        url = Chibi_url( url )
        api_url = Chibi_url(
            "https://jumpg-webapi.tokyo-cdn.com/api/title_detail" )
        api_url = api_url + { 'title_id': url.base_name }
        url = api_url
        super().__init__( url, *args, **kw )

    def download( self, path ):
        raise NotImplementedError

    @property
    def name( self ):
        try:
            return self._title
        except AttributeError:
            self.load_soup()
            return self._title

    @property
    def episodes( self ):
        try:
            return self._episodes
        except AttributeError:
            self.load_soup()
            return self._episodes

    def load_soup( self ):
        page = self.get( self.url, )
        page.close()
        chapters = re_chapter.findall( page.text )
        chapters_titles = re_chapter_title.findall( page.text )
        title = re_title.findall( page.text )
        if title:
            title = title[0].strip()

        # url_base = "https://mangaplus.shueisha.co.jp/viewer/{id}"
        url_base = Chibi_url(
            "https://jumpg-webapi.tokyo-cdn.com/api/manga_viewer" )
        params = { 'chapter_id': "", 'split': "yes", 'img_quality': 'high' }

        self._episodes = []
        for chapter_id, chapter_title in zip( chapters, chapters_titles ):
            params[ "chapter_id" ] = chapter_id
            self._episodes.append( Episode.from_site(
                site=self, url=url_base + params,
                title=chapter_title ) )



import logging
import re
import shutil

from bs4 import BeautifulSoup
from chibi.file import Chibi_path
from chibi.atlas import Chibi_atlas

from chibi_dl.site.base.site import Site
from chibi_requests import Chibi_url
from .regex import re_image


logger = logging.getLogger( "chibi_dl.sites.ehentai.episode" )


class Image( Site ):
    @classmethod
    def can_proccess( cls, url ):
        if re_image.match( str( url ) ):
            return cls( url )

    def parse_info( self ):
        image = Chibi_url( self.soup.find(
            "section", id="image-container" ).img.get( 'src' ) )
        return Chibi_atlas( image=image, )

    @property
    def image( self ):
        return self.info.image

    def download( self, path ):
        if path.is_a_folder:
            path += self.image.base_name
        logger.info( f"descargnado {path.base_name} en {path.dir_name}" )
        self.image.download( path )
        return path



import itertools
import logging
import time

from bs4 import BeautifulSoup
from chibi.file.temp import Chibi_temp_path

from .episode import Episode
from chibi_dl.site.base.site import Site


logger = logging.getLogger( "chibi_dl.sites.tmo_fans.serie" )


class Serie( Site ):
    def __init__( self, url, *args, **kw ):
        super().__init__( url, *args, **kw )

    def download( self, path ):
        serie_path = ( path + self.name ).made_safe()
        serie_path.mkdir()

        logger.info( "iniciando descarga de la serie '{}' de {}".format(
            self.name, self.url ) )
        for episode in self.episodes:
            episode_path = serie_path + episode.file_name
            if ( episode_path.exists ):
                logger.info( (
                    "ignorando el episodio {} se encontro "
                    "en el destino" ).format( episode.title ) )
                continue
            downlaod_folder = Chibi_temp_path()
            try:
                episode.download( downlaod_folder )
            except Exception as e:
                logger.exception(
                    "paso un problema cuando intento de "
                    "descargar el episodio" )
                continue
            episode.compress( serie_path, downlaod_folder )
            logger.info(
                'termino de descargar el episodio esperando '
                '10 seguncods para el siguiente' )
            time.sleep( 10 )

    @property
    def name( self ):
        try:
            return self._title
        except AttributeError:
            self.load_soup()
            return self._title

    @property
    def episodes( self ):
        try:
            return self._episodes
        except AttributeError:
            self.load_soup()
            return self._episodes

    def load_soup( self ):
        page = self.get( self.url, )
        soup = BeautifulSoup( page.content, 'html.parser' )
        try:
            self._title = "".join(
                soup.select( ".element-title.my-2" )[0].find_all(
                    string=True, recursive=False ) ).strip()
            self.load_episodes( soup )
        except Exception as e:
            import pdb
            pdb.post_mortem( e.__traceback__ )
            raise

    def load_episodes( self, soup ):
        self._episodes = []
        if "one_shot" in self.url:
            self.load_one_shot( soup )
            return
        else:
            chapter_container = soup.find(
                "div", { 'class': "card chapters" } )
            chapter_container_hidden = chapter_container.find(
                "div", id="chapters-collapsed" )

            if not chapter_container_hidden:
                chapters = chapter_container.ul.find_all(
                    "li", recursive=False )
            else:
                chapters = itertools.chain(
                    chapter_container.ul.find_all( "li", recursive=False ),
                    chapter_container_hidden.find_all( "li", recursive=False )
                )

            for chapter in chapters:
                links = chapter.select(
                    "div.card.chapter-list-element" )[0].find_all( 'a' )
                fansub = links[0].text
                title = chapter.find( "h4" ).a.text
                url = links[0].find_next(
                    "span", { "class": "fas fa-play fa-2x"}
                ).parent.get( 'href' )

                self._episodes.append(
                    Episode.from_site(
                        site=self, url=url, fansub=fansub, title=title ) )

    def load_one_shot( self, soup ):
        chapters = soup.find( "div", { "class": "card chapter-list-element" } )
        chapters = chapters.ul.find_all( "li", recursive=False )
        for i, chapter in enumerate( chapters ):
            parts = chapter.div.find_all( "div", recursive=False )
            url = parts[-1].a.get( "href" ).strip()

            self._episodes.append(
                Episode.from_site(
                    site=self, url=url, fansub=parts[0].text.strip(),
                    title=str( i ) ) )



from marshmallow import Schema, pre_load, fields, EXCLUDE, post_load
from chibi_marshmallow import fields as chibi_fields


class Torrent( Schema ):
    add_at = chibi_fields.Timestamp( data_key='added' )
    file_size = fields.Integer( data_key='fsize' )
    size = fields.Integer( data_key='tsize' )
    name = fields.String()
    hash = fields.String()


class Metadata( Schema ):
    category = chibi_fields.String_lower()
    expunged = fields.Bool()
    images_len = fields.Integer( data_key='filecount' )
    images_size = fields.Integer( data_key='filesize' )
    id = fields.Integer( data_key='gid' )
    upload_at = chibi_fields.Timestamp( data_key='posted' )
    rating = fields.Float()
    title = fields.String()
    titles__jpn = fields.String( data_key='title_jpn' )
    token = fields.String()
    torrent_count = fields.Integer( data_key='torrentcount' )
    torrents = fields.Nested( Torrent, many=True )
    uploader = fields.String()

    cover_url = fields.String( data_key='thumb' )
    archiver_key = fields.String()

    tags = fields.List( chibi_fields.String_lower() )

    @pre_load
    def ignore_gmetadata( self, data, *args, many=False, **kw ):
        if many:
            return data[ 'gmetadata' ]
        return data[ 'gmetadata' ][0]

    @post_load
    def rename_title_jpn( self, data, *args, **kw ):
        title_jpn = data.pop( 'titles__jpn' )
        data[ 'alternative_titles' ] = dict( jpn=title_jpn )
        return data



import time

from chibi.file import Chibi_path
from chibi.file.temp import Chibi_temp_path
from chibi_dl.site import Site
from unittest import TestCase
from unittest import TestCase, skip
from vcr_unittest import VCRTestCase


class Test_base:
    def _get_vcr_kwargs( self, **kw ):
        result = super()._get_vcr_kwargs( **kw )
        result[ 'ignore_localhost' ] = True
        return result

    def test_site_should_get_all_the_links_in_a_page( self ):
        links = list( self.site.links )
        self.assertTrue( self.site.links )

    def test_without_domain_should_return_with_current( self ):
        result = self.site.build_link( '/hello' )
        expected = 'https://www.google.com/hello'
        self.assertEqual( result, expected )

    def test_with_domain_should_return_with_the_domain( self ):
        result = self.site.build_link( '/hello', domain='https://other.com' )
        expected = 'https://other.com/hello'
        self.assertEqual( result, expected )

    def test_if_the_url_have_domain_should_return_the_url( self ):
        result = self.site.build_link( 'https://google.com/hello' )
        expected = 'https://google.com/hello'
        self.assertEqual( result, expected )


class Test_google( Test_base, VCRTestCase ):
    def setUp( self ):
        super().setUp()
        self.site = Site( url='https://www.google.com' )



import logging
import time

import requests
from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.firefox.options import Options
from selenium import webdriver
from selenium.webdriver import DesiredCapabilities
from selenium.webdriver.common.by import By
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import undetected_chromedriver as uc

from .exceptions import Cannot_pass_cloud_flare, Cannot_login
from .regex import re_show, re_follow, re_pending, re_read
from chibi_dl.site.base.site import Site as Site_base


logger = logging.getLogger( "chibi_dl.sites.tmo_fans" )


class Site( Site_base ):
    def __init__( self, *args, **kw ):
        self._cloud_flare_passed = False
        super().__init__( *args, **kw )

    def build_session( self ):
        self._session = requests.session()

    @property
    def cloud_flare_passed( self ):
        return (
            self._cloud_flare_passed
            or ( self.parent and self.parent.cloud_flared_passed ) )

    @cloud_flare_passed.setter
    def cloud_flare_passed( self, value ):
        self._cloud_flare_passed = value
        if self.parent:
            self.cloud_flare_passed = value

    def cross_cloud_flare( self ):
        if not self.cloud_flare_passed:
            logger.info( f"obteniendo {self.url} usando el navegador" )
            self.firefox.get( self.url )
            logger.info( "esperando 10 segundos a que pase cloud flare" )
            time.sleep( 10 )
            self.cookies = self.firefox.get_cookies()
            self.user_agent = self.firefox.execute_script(
                "return navigator.userAgent;" )


            if self.get( url=self.url ).ok:
                logger.info( "se pueden descargar las imagenes de TMOFans" )
                self.cloud_flare_passed = True
            else:
                raise Cannot_pass_cloud_flare

    @property
    def cookies( self ):
        if self.parent:
            return self.parent.cookies
        try:
            return self._cookies
        except AttributeError:
            return None

    @cookies.setter
    def cookies( self, value ):
        if self.parent:
            self.parent.cookies = value
        else:
            self._cookies = value
            self.session.cookies.clear()
            for cookie in value:
                self.session.cookies.set( cookie[ 'name' ], cookie[ 'value' ] )

    @property
    def firefox( self ):
        return super().browser
        if self.parent:
            return self.parent.firefox
        try:
            return self._firefox
        except AttributeError:
            options = Options()

            """
            profile = webdriver.FirefoxProfile()
            profile.set_preference( "dom.webdriver.enabled", False )
            profile.set_preference( 'useAutomationExtension', False )
            profile.set_preference(
                "general.useragent.override",
                "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) "
                "AppleWebKit/537.36 (KHTML, like Gecko) "
                "Chrome/83.0.4103.61 Safari/537.36" )
            profile.update_preferences()
            desired = DesiredCapabilities.FIREFOX
            """


            """
            #options.headless = True
            logger.info( "abriendo firefox" )
            #self._firefox = webdriver.Firefox( options=options )
            self._firefox = webdriver.Firefox(
                firefox_profile=profile,
                desired_capabilities=desired
            )
            logger.info( "abrieno chrome" )
            self._firefox.execute_script(
                "Object.defineProperty(navigator, 'webdriver',"
                "{get: () => undefined})" )
            navigator_webdriver = self._firefox.execute_script(
                "return navigator.webdriver;" )
            logger.info( f"firefox.navigator.webdriver: {navigator_webdriver}" )
            """
            desire_capabilities = DesiredCapabilities.CHROME
            desire_capabilities[ 'pageLoadStrategy' ] = 'eager'
            logger.info( "iniciando chrome" )
            a = uc.Chrome( desired_capabilities=desire_capabilities );
            self._firefox = a
            return self._firefox

    def __del__( self ):
        if self.parent is None:
            try:
                self._firefox.quit()
                logger.info( "cerrando firefox" )
            except AttributeError:
                pass
        super().__del__()


class TMO_fans( Site ):
    def __init__( self, *args, user=None, password=None, **kw ):
        super().__init__( 'https://tmofans.com/login', *args, **kw )
        self.series = []
        self.user = user
        self.password = password
        self._login_ok = False

    def append( self, url ):
        from .serie import Serie
        self.cross_cloud_flare()
        if re_show.match( url ):
            self.series.append( Serie( url, parent=self ) )
        elif re_follow.match( url ):
            self.login()
            for l in self.get_all_follow():
                self.append( l )
        elif re_pending.match( url ):
            self.login()
            for l in self.get_all_pending():
                self.append( l )
        elif re_read.match( url ):
            self.login()
            for l in self.get_all_read():
                self.append( l )
        else:
            logger.error(
                "la url {} no se pudo identificar como serie".format( url ) )

    def login( self ):
        if self._login_ok:
            return
        if not self.cloud_flare_passed:
            self.cross_cloud_flare()

        email = self.firefox.find_element( By.ID, "email" )
        password = self.firefox.find_element( By.ID, "password" )
        submit = self.firefox.find_element(
            By.XPATH,
            "/html/body/div[1]/main/div/div/div/div[1]/form/div[4]"
            "/div[1]/button" )

        email.send_keys( self.user )
        password.send_keys( self.password )

        submit.click()
        logger.info( "esperando 10 segundos a que termine el login" )
        time.sleep( 10 )
        if self.firefox.current_url == self.url:
            raise Cannot_login
        else:
            self._login_ok = True
            self.cookies = self.firefox.get_cookies()
            self.user_agent = self.firefox.execute_script(
                "return navigator.userAgent;" )

        """
        page = self.get( self.url )
        soup = BeautifulSoup( page.content, 'html.parser' )
        form = soup.find( "form", { "class": "form-horizontal" } )
        token = form.find( "input", dict( name="_token" ) )[ "value" ]

        payload = dict(
            email=self.user, password=self.password, remember="on",
            _token=token )

        response = self.session.post(
            self.url, data=payload, headers={
                "Referer": str( self.url ),
                "Content-type": "application/x-www-form-urlencoded" } )

        if response.ok and response.url == self.url:
            raise Cannot_login
        else:
            self._login_ok = True
        """

    def get_all_follow( self ):
        page_number = 0
        url = "https://tmofans.com/profile/follow"
        while( True ):
            page_number += 1
            page = self.get( url=url, params={ "page": page_number } )
            soup = BeautifulSoup( page.content, 'html.parser' )

            links = self.get_manga_links( soup )
            if not links:
                return
            for l in links:
                yield l

    def get_all_pending( self ):
        page_number = 0
        url = "https://tmofans.com/profile/pending"
        while( True ):
            page_number += 1
            page = self.get( url=url, params={ "page": page_number } )
            soup = BeautifulSoup( page.content, 'html.parser' )

            links = self.get_manga_links( soup )
            if not links:
                return
            for l in links:
                yield l

    def get_all_read( self ):
        page_number = 0
        url = "https://tmofans.com/profile/read"
        while( True ):
            page_number += 1
            page = self.get( url=url, params={ "page": page_number } )
            soup = BeautifulSoup( page.content, 'html.parser' )

            links = self.get_manga_links( soup )
            if not links:
                return
            for l in links:
                yield l

    def get_manga_links( self, soup ):
        result = []
        for a in soup.find_all( "a" ):
            if "library/" in a[ "href" ]:
                result.append( a[ "href" ].strip() )
        return result

    def i_can_proccess_this( self, url ):
        regex = ( re_show, re_pending, re_read, re_follow )
        return any( ( r.match( url ) for r in regex ) )



import logging

import ffmpeg
import m3u8
from chibi.atlas import loads, Chibi_atlas
from chibi.file import Chibi_path
from chibi_dl.config import configuration
from pymkv import MKVFile, MKVTrack

from .exceptions import Cannot_find_subtitles, Episode_not_have_media
from .site import Site
from .subtitles import Subtitle


logger = logging.getLogger( "chibi_dl.sites.crunchyroll.episode" )


class Episode( Site ):
    def parse_info( self ):
        result = Chibi_atlas()
        result.media_metadata = self._parse_media_metadata()
        result.number = self._parse_episode_number()
        result.serie_title = result.media_metadata.series_title
        result.title = result.media_metadata.episode_title
        result.subtitles = self._parse_subtitles( result.title )
        result.playlist = self._parse_playlist()
        return result

    def _parse_media_metadata( self ):
        soup = self.soup
        return soup.config_Config.default_preload.media_metadata

    def _parse_subtitles( self, title ):
        subtitles = self.soup.config_Config.default_preload.subtitles.subtitle
        if isinstance( subtitles, dict ):
            subtitles = [ subtitles ]
        return [
            Subtitle.from_info(
                self.url + s.link, name=title, parent=self, **s )
            for s in subtitles ]

    def _parse_episode_number( self ):
        media = self.soup.config_Config.default_preload.media_metadata
        number = media.episode_number
        if number:
            try:
                return int( number )
            except ValueError:
                try:
                    return float( number )
                except ValueError:
                    return number

    @property
    def subtitles( self ):
        return self.info.subtitles

    @property
    def media_id( self ):
        result = self.url.url.base_name.rsplit( '-', 1 )[-1]
        return result

    @property
    def url_data( self ):
        video_quality = configuration.chibi_dl.crunchyroll.video_quality
        video_format = configuration.chibi_dl.crunchyroll.video_format

        url_data = self.url + "http://www.crunchyroll.com/xml/?req=RpcApiVideoPlayer_GetStandardConfig"

        url_data = url_data + dict(
            media_id=self.media_id, current_page=self.url,
            video_quality=video_quality, video_format=video_format, )

        return url_data

    def load( self ):
        url = self.url_data
        response = self.get( url=url )
        self._response = response

    @property
    def stream( self ):
        resolution = configuration.chibi_dl.crunchyroll.resolution
        stream_with_my_resolution = filter(
            lambda x: x.stream_info.resolution == resolution,
            self.info.playlist )

        try:
            first_stream = next( stream_with_my_resolution )
        except StopIteration:
            resolutions = [ p.stream_info.resolution for p in self.info.playlist ]
            logger.exception(
                "no se encontro la resolucion {} resoluciones "
                "encontradas {}".format( resolution, resolutions ) )
            raise
        return first_stream

    def download_stream( self, path ):
        if path.is_a_folder:
            path += self.file_name
        stream = ffmpeg.input( self.stream.uri )
        stream = ffmpeg.output(
            stream, path, loglevel="quiet", codec="copy", )
        logger.info(
            f'inicia la descarga del video "{self.name}" en "{path}"'
        )
        ffmpeg.run( stream )
        return path

    def __str__( self ):
        return self.name

    def __repr__( self ):
        return f"Episode( {self} )"

    def _parse_playlist( self ):
        stream_info = self.soup.config_Config.default_preload.stream_info
        if 'error' in stream_info:
            raise Episode_not_have_media( stream_info )
        straming_list = m3u8.load( stream_info.file )
        return straming_list.playlists

    def download( self, path, download_subtitles=True ):
        logger.info( "inicio de la descarga de subtitulos"  )
        if download_subtitles:
            for subtitles in self.subtitles:
                subtitles.download( path )
        return self.download_stream( path )

    def pack( self, path ):
        mkv = MKVFile()
        mkv.add_track( MKVTrack( path + self.file_name, track_id=0 ) )
        mkv.add_track( MKVTrack( path + self.file_name, track_id=1 ) )
        for subtitle in self.subtitles:
            mkv.add_track( subtitle.to_mkv_track( path ) )
        output = path + self.file_name_mkv
        logger.info( "inicia el empaquetado a '{}'".format( self.file_name ) )
        mkv.mux( output, silent=False )
        return output

    @property
    def serie_title( self ):
        title = self.info.media_metadata.series_title
        title = title.replace( "/", " " )
        return title



    @property
    def name( self ):
        title = self.serie_title.replace( 'Season ', 'S' )
        if self.info.number:
            return f"{title} - {self.info.number}"
        else:
            return "{title}"

    @property
    def file_name( self ):
        ext = 'm4a'
        resolution = configuration.chibi_dl.crunchyroll.resolution
        resolution = "[{}x{}]".format( *resolution )
        result = f"{self.name} {resolution}.{ext}"
        return Chibi_path( result ).made_safe()

    @property
    def file_name_mkv( self ):
        ext = 'mkv'
        resolution = configuration.chibi_dl.crunchyroll.resolution
        resolution = "[{}x{}]".format( *resolution )
        result = f"{self.name} {resolution}.{ext}"
        return Chibi_path( result ).made_safe()



import base64
import logging
import zlib

import pycountry
from chibi.atlas import loads, Chibi_atlas
from chibi.atlas import loads
from chibi.file import Chibi_path
from chibi.parser import to_bool
from pymkv import MKVTrack

from .external.aes import aes_cbc_decrypt
from .external.utils import obfuscate_key, bytes_to_intlist, intlist_to_bytes
from .site import Site


logger = logging.getLogger( "chibi_dl.sites.crunchyroll.subtitle" )


class Subtitle( Site ):
    @classmethod
    def from_info( cls, url, parent=None, **data ):
        data = Chibi_atlas( data )
        result = cls( url, parent=parent )
        result.info.title = data.title
        result.info.name = data.name
        result.info.id = data.id
        result.info.default = data.default
        return result

    def parse_info( self ):
        data = self.soup.subtitle
        return data

    @property
    def subtitle_xml( self ):
        try:
            return self._subtitle_xml
        except:
            data = self.soup.subtitle
            data = loads( self.decrypt_subtitles(
                data.data, data.iv, data.id ) )
            self._subtitle_xml = data.subtitle_script
            return self._subtitle_xml

    def download( self, path ):
        if path.is_a_folder:
            path += self.file_name
        f = path.open()
        f.write( self.ass )
        logger.info( f'write subtitulo "{path}"' )
        return path

    @property
    def default( self ):
        return to_bool( self.info.default )

    @property
    def lang( self ):
        return "{}-{}".format(
            self.subtitle_xml.lang_code[:2], self.subtitle_xml.lang_code[-2:] )

    @property
    def lang_ISO_639_2( self ):
        try:
            return pycountry.languages.get(
                alpha_2=self.lang[:2] ).bibliographic
        except AttributeError:
            return pycountry.languages.get( alpha_2=self.lang[:2] ).alpha_3

    @property
    def name( self ):
        return f"{self.info.name}.{self.lang}"

    @property
    def file_name( self ):
        ext = 'ass'
        result = Chibi_path( "{name}.{ext}".format( name=self.name, ext=ext ) )
        result = result.made_safe()
        return result

    def __str__( self ):
        return self.name

    def __repr__( self ):
        return f"Subtitle( {self} )"

    def to_mkv_track( self, path ):
        track = MKVTrack(
            path + self.file_name, language=self.lang_ISO_639_2,
            default_track=self.default, track_name=self.data.title )
        return track

    @property
    def ass( self ):
        data = self.subtitle_xml
        output = ''

        def ass_bool(strvalue):
            assvalue = '0'
            if strvalue == '1':
                assvalue = '-1'
            return assvalue

        output = '[Script Info]\n'
        output += f'Title: {data.title}\n'
        output += 'ScriptType: v4.00+\n'
        output += f'WrapStyle: {data.wrap_style}\n'
        output += f'PlayResX: {data.play_res_x}\n'
        output += f'PlayResY: {data.play_res_y}\n'
        output += "[V4+ Styles]\n"
        output += (
            "Format: Name, Fontname, Fontsize, PrimaryColour, SecondaryColour,"
            " OutlineColour, BackColour, Bold, Italic, Underline, StrikeOut,"
            " ScaleX, ScaleY, Spacing, Angle, BorderStyle, Outline, Shadow,"
            " Alignment, MarginL, MarginR, MarginV, Encoding" )
        if not isinstance( data.styles.style, list ):
            data.styles.style = [ data.styles.style ]
        for style in data.styles.style:
            output += 'Style: ' + style['name']
            output += ',' + style['font_name']
            output += ',' + style['font_size']
            output += ',' + style['primary_colour']
            output += ',' + style['secondary_colour']
            output += ',' + style['outline_colour']
            output += ',' + style['back_colour']
            output += ',' + ass_bool(style['bold'])
            output += ',' + ass_bool(style['italic'])
            output += ',' + ass_bool(style['underline'])
            output += ',' + ass_bool(style['strikeout'])
            output += ',' + style['scale_x']
            output += ',' + style['scale_y']
            output += ',' + style['spacing']
            output += ',' + style['angle']
            output += ',' + style['border_style']
            output += ',' + style['outline']
            output += ',' + style['shadow']
            output += ',' + style['alignment']
            output += ',' + style['margin_l']
            output += ',' + style['margin_r']
            output += ',' + style['margin_v']
            output += ',' + style['encoding']
            output += '\n'

        output += (
            "\n[Events]\n"
            "Format: Layer, Start, End, Style, Name, MarginL, "
            "MarginR, MarginV, Effect, Text\n"
        )

        if not isinstance( data.events.event, list ):
            data.events.event = [ data.events.event ]
        for event in data.events.event:
            output += 'Dialogue: 0'
            output += ',' + event['start']
            output += ',' + event['end']
            output += ',' + event['style']
            output += ',' + event['name']
            output += ',' + event['margin_l']
            output += ',' + event['margin_r']
            output += ',' + event['margin_v']
            output += ',' + event['effect']
            output += ',' + event['text']
            output += '\n'

        return output

    def decrypt_subtitles( self, data, iv, _id ):
        data = bytes_to_intlist( base64.b64decode( data ) )
        iv = bytes_to_intlist( base64.b64decode( iv ) )
        id = int( _id )
        key = obfuscate_key( id )
        decrypted_data = intlist_to_bytes(
            aes_cbc_decrypt( data, key, iv ) )
        return zlib.decompress( decrypted_data )



import logging

from .regex import re_episode, re_main
from chibi_dl.site.base.site import Site as Site_base
from chibi_requests import Chibi_url
from chibi.metaphors import Book
from chibi.metaphors.book import End_book
from chibi.atlas import Chibi_atlas


logger = logging.getLogger( "chibi_dl.sites.nhentai" )


class Site( Site_base ):
    pass


class Nhentai( Site ):
    def __init__( self, url=None, *args, book=None, **kw ):
        if url is None:
            url = Chibi_url( 'https://nhentai.net' )
        super().__init__( url, *args, book=book, **kw )
        self.enable_full_scan = False
        self.processing_order = [ self.episode_class ]

    def append( self, url ):
        result = super().append( url )
        if result:
            return result
        url = Chibi_url( url )

        if re_main.match( str( url.url ) ):
            self.enable_full_scan = True
        return result

    def __iter__( self ):
        if self.urls:
            for url in self.urls:
                yield url
        if self.enable_full_scan:
            for galerie in self.info.galeries:
                yield galerie
            for page in self.pages:
                for galerie in page.info.galeries:
                    yield galerie

    @property
    def pages( self ):
        book = Book(
            total_elements=self.last_page, page_size=1,
            page=self.current_page, offset_dict={ 'page': 'page' } )
        for page in book:
            yield type( self )( self.url + page, parent=self )

    def parse_info( self ):
        galeries = self.soup.find_all( "div", { "class": "gallery" } )
        links = [
            self.episode_class(
                self.url + g.a.attrs[ 'href' ], parent=self )
            for g in galeries ]
        return Chibi_atlas( galeries=links )

    @property
    def current_page( self ):
        return int( self.url.params.get( "page", 1 ) )

    @property
    def last_page( self ):
        page = self.soup.find( "a", { "class": "last" } ).attrs[ 'href' ]
        return int( page.split( '=' )[1] )

    @property
    def episode_class( self ):
        from .episode import Episode
        return Episode

    @classmethod
    def can_proccess( cls, url ):
        if re_main.match( str( url ) ):
            return cls( url )

    def __bool__( self ):
        return bool( self.url )

    def i_can_proccess_this( self, url ):
        regex = ( re_episode, re_main )
        return any( ( r.match( url ) for r in regex ) )



import logging
import re
import shutil

from chibi.file import Chibi_path

from chibi_dl.site.base.site import Site


logger = logging.getLogger( "chibi_dl.sites.manga_plus.episode" )


class Episode( Site ):
    def download( self, path ):
        logger.info(
            "iniciando la descarga de las {} imagenes del capitulo {}".format(
                len( self.images_urls ), self.number,
            ) )
        raise NotImplementedError

    def compress( self, path_ouput, path_input, format="zip" ):
        logger.info( "comprimiendo capitulo usando {}".format( format ) )
        result = Chibi_path( shutil.make_archive(
            path_ouput + self.number, format, path_input ) )
        expected = result.replace_extensions( "cbz" )
        result.move( expected )
        return expected

    @property
    def file_name( self ):
        return "{}.{}".format( self.number, "cbz" )

    @property
    def number( self ):
        re_float = re.compile( r"[-+]?([0-9]*\.[0-9]+|[0-9]+)" )
        return re_float.findall( self.title )[0]

    @property
    def images_urls( self ):
        try:
            return self._images_urls
        except AttributeError:
            self.load_soup()
            return self._images_urls

    def load_soup( self ):
        page = self.get( self.url )
        raise NotImplementedError
        return page.text

    def enumerate_images_urls( self ):
        for i, url in enumerate( self.images_urls ):
            ext = url.rsplit( '.', 1 )[-1]
            yield "{}.{}".format( i, ext ), url



import os
try:
    from setuptools import setup, find_packages
except:
    from distutils.core import setup, find_packages

here = os.path.abspath( os.path.dirname( __file__ ) )
README = open(os.path.join( here, 'README.rst' ) ).read()

setup(
    name='chibi_dl',
    version='0.1.3',
    description='',
    long_description=README,
    license='',
    author='dem4ply',
    author_email='',
    packages=find_packages(),
    install_requires=[
        'chibi>=0.11.10', "chibi_requests>=0.1.1",
        'selenium>=3.141.0',
        'undetected-chromedriver>=3.4.6',
    ],
    dependency_links = [],
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
    ],
    entry_points = {
        'console_scripts': [
            'chibi_dl=chibi_dl.main:main'
        ],
    }
)



#!/usr/bin/env python3
import json
import sys
import logging
import random
from argparse import ArgumentParser

from chibi.file import Chibi_path
from chibi.config import basic_config, load as load_config

from chibi_dl.site import Site


logger_formarter = '%(levelname)s %(name)s %(asctime)s %(message)s'


parser = ArgumentParser(
    description="descarga mangas", fromfile_prefix_chars='@'
)

parser.add_argument(
    "sites", nargs='+', metavar="site",
    help="urls de las series que se quieren descargar" )

parser.add_argument(
    "--only_print", dest="only_print", action="store_true",
    help="define si silo va a imprimir la lista de links o episodios"
)

parser.add_argument(
    "--only_metadata", dest="only_metadata", action="store_true",
    help="se define si solo se queire recolectar los datos y no descargar"
)

parser.add_argument(
    "--only_links", dest="only_print_links", action="store_true",
    help="si se usa solo imprimira las urls"
)

parser.add_argument(
    "--random", dest="random", action="store_true",
    help="procesa las urls en un orden aleatorio"
)

parser.add_argument(
    "--log_level", dest="log_level", default="INFO",
    help="nivel de log",
)

parser.add_argument(
    "-config_site", type=Chibi_path, dest="config_site",
    help="python, yaml o json archivo con el usuario y password de cada sitio"
)


def main():
    args = parser.parse_args()
    basic_config( args.log_level )

    if args.config_site:
        load_config( args.config_site )

    for site in args.sites:
        site = Site( url=site )
        for link in site.links:
            print( link )



import itertools
import logging

from bs4 import BeautifulSoup
from chibi.file.temp import Chibi_temp_path
from chibi.atlas import Chibi_atlas

from .episode import Episode
from .regex import re_episodes
from .regex import re_show, re_video, re_lenguage, re_csrf_token
from .site import Site
from chibi_dl.site.crunchyroll.exceptions import Episode_not_have_media
from chibi.atlas import Chibi_atlas


logger = logging.getLogger( "chibi_dl.sites.crunchyroll.show" )


class Serie( Site ):

    @classmethod
    def can_proccess( cls, url ):
        if re_show.match( str( url ) ):
            return cls( url )

    def parse_info( self ):
        result = Chibi_atlas()
        result.title = self.soup.select( "h1.ellipsis" )[0].text.strip()
        description = self.soup.find( "p", { "class": "description" } )
        result.description = description.find(
            "span", { "class": "more" } ).text.strip()

        seasons = self.soup.find( "ul", { "class": "list-of-seasons cf" } )
        result.seasons = []
        for season in seasons.find_all( "li", recursive=False ):
            _season = Chibi_atlas()
            result.seasons.append( _season )
            _season.name = season.a.text.strip()
            _season.episodes = []
            episodes = season.find_all( "a", { "class": "episode" } )
            for episode in episodes:
                e = Episode(
                    url=self.url + episode.attrs[ "href" ], parent=self )
                _season.episodes.append( e )
        return result

    def __iter__( self ):
        return itertools.chain.from_iterable(
            season.episodes for season in self.info.seasons )

    def download( self, path ):
        logger.info(
            'iniciando descarga de la serie "{}" de "{}"'.format(
                self.title, self.url ) )
        path += self.title
        path = path.made_safe()
        path.mkdir()
        for episode in self.episodes:
            episode_path = path + episode.file_name_mkv
            if ( episode_path.exists ):
                logger.info( (
                    "ignorando el episodio {} se encontro "
                    "en el destino" ).format( episode.name ) )
                continue
            temp_folder = Chibi_temp_path()
            try:
                episode.download( temp_folder )
            except Episode_not_have_media:
                logger.error( (
                    "el episodio {} no esta disponible posiblemente el "
                    "login no esta correcto" ).format(
                        episode.name ) )
                continue
            mkv = episode.pack( temp_folder )
            mkv.move( path )

    @property
    def metadata( self ):
        try:
            raise NotImplementedError
            return self._metadata
        except AttributeError:
            self._metadata = Chibi_atlas()

    @property
    def title( self ):
        return self.info.title

    @property
    def episodes( self ):
        try:
            return self._episodes
        except AttributeError:
            self._episodes = list(
                itertools.chain.from_iterable(
                    season.episodes for season in self.info.seasons ) )
            return self._episodes

    def load_episodes( self ):
        page = self.get( self.url, )
        soup = page.native

        self._episodes = []
        episodes_links = re_episodes.findall( page.native.text )
        episodes = page.native.find_all(
            'a',
            **{ 'class': 'portrait-element block-link titlefix episode' }
        )
        for episode in episodes:
            url = self.url + episode.attrs[ 'href' ]
            self._episodes.append( Episode.from_site(
                self, url=url ) )
        """
        for episode_link, episode_type in episodes_links:
            episode_link = episode_link.rsplit( '/', 1 )[1]
            episode_url = "{url}/{episode}".format(
                url=self.url, episode=episode_link )
            if episode_link:
                self._episodes.append( Episode.from_site(
                    self, url=episode_url ) )
        """



from chibi_requests.chibi_url import Chibi_url
from chibi_requests.other import Force_json
from chibi_dl.site.ehentai.serializers import Metadata


class Metadata_response( Force_json ):
    serializer = Metadata


metadata = Chibi_url(
    "https://api.e-hentai.org/api.php", response_class=Metadata_response )



import logging

from .regex import re_show
from chibi_dl.site.base.site import Site as Site_base


logger = logging.getLogger( "chibi_dl.sites.manga_plus" )


class Site( Site_base ):
    pass


class Manga_plus( Site ):
    def __init__( self, *args, **kw ):
        super().__init__(
            'https://mangaplus.shueisha.co.jp/updates', *args, **kw )
        self.series = []

    def append( self, url ):
        from .serie import Serie
        if re_show.match( url ):
            self.series.append( Serie( url, parent=self ) )
        else:
            logger.error(
                "la url {} no se pudo identificar como serie".format( url ) )

    def i_can_proccess_this( self, url ):
        regex = ( re_show, )
        return any( ( r.match( url ) for r in regex ) )
